/*
 * Copyright (C) 2020 Samsung Electronics Co., Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef __TELF_H
#define __TELF_H
#include <elf.h>
#include <stdbool.h>
#include <sys/types.h>

#define ERR_MSG_LEN 1024

enum TEU_STATUS  {TEU_NOINIT = 0,
                  TEU_FILE,
	          TEU_MEMORY};

struct teu_elf {
	enum TEU_STATUS status;
	int fd;
	char *ident;
	off64_t saved_offset;
	Elf64_Ehdr ehdr;
	char last_error[ERR_MSG_LEN];
	void *data;
	off64_t data_size;
};

typedef struct teu_elf Elf;

#ifdef __cplusplus
extern "C" {
#endif

char *teu_errmsg(Elf *elf);

bool teu_iself(Elf *elf);

uint8_t teu_class(Elf *elf);

bool teu_internal_getehdr(Elf *elf, Elf64_Ehdr *ehdr);

bool teu_begin(int fd, Elf *elf);

bool teu_begin_memory(void *memory, Elf *elf);

bool teu_getehdr(Elf *elf, Elf64_Ehdr *ehdr);

bool teu_getphdr(Elf *elf, uint64_t index, Elf64_Phdr *phdr);

bool teu_getshdr(Elf *elf, uint64_t index, Elf64_Shdr *shdr);

char *teu_secname(Elf *elf, Elf64_Shdr *shdr);

void *teu_getsdata(Elf *elf, Elf64_Shdr *shdr);

bool teu_getsym(Elf *elf, Elf64_Shdr *shdr, int index, void *data, Elf64_Sym *sym);

char *teu_strptr(Elf *elf, int sindex, off64_t offset);

bool teu_close(Elf *elf);

bool teu_getdata(Elf *elf, off64_t offset, uint64_t size, void *data);

off64_t teu_getnote(Elf *elf, void *data, off64_t offset, Elf64_Nhdr *nhdr, off64_t *name_offset, off64_t *desc_offset);

#ifdef __cplusplus
}
#endif

#endif  // __TELF_H
